----------------------------------------
-- 以下是引擎提供接口
----------------------------------------
-- #pragma mark  -- 游戏相关能力

--[[
游戏已经准备好
@param context 游戏上下文
@param seqId 请求序列号
--]]
function appGameReady(context, seqId)
    print_Debug("------- Lua Receive NativeBridge appGameReady context = " .. context .. " seqId = " .. seqId)
    -- onPKGameReady()
    SendMessage(MsgID_Native_Bridge_Exit_AppGameReady)
end

--[[
 游戏退出
 @param context 游戏退出
 @param seqId 我的游戏进行到哪个阶段
 --]]
function appGameExit(context, seqId)
    print_Debug("------- Lua Receive NativeBridge appGameExit context = " .. context .. " seqId = " .. seqId)
    SendMessage(MsgID_Native_Bridge_Exit_Game)
end


----------------------------------------
-- #pragma mark  -- app事件通知

--[[
 app发生某事件通知游戏
 @param context 游戏上下文
 @param seqId 请求序列号
 @param event 事件ID
 @param args 参数列表
 --]]
function appSentEventToGame(context, seqId, event, args)
    print_Debug("------- Lua Receive NativeBridge appSentEventToGame context = " .. context .. " seqId = " .. seqId .. " event = " .. event .. " args = " .. args)
    SendMessage(MsgID_Native_Bridge_SendGameEvent, context ,event, args, seqId)
end


----------------------------------------
-- #pragma mark  -- app基础能力
IsAPPInBackground = false
--[[
 app进入前台
 @param context 游戏上下文
 @param seqId 请求序列号
 --]]
function appEnterForeground(context, seqId)
    print_Debug("------- Lua Receive NativeBridge appEnterForeground context = " .. context .. " seqId = " .. seqId)
    SendMessage(MsgID_Native_Bridge_EnterForeground)
    IsAPPInBackground = false
    Audio.resumeAllByAutoResumeFlag()
end

--[[
 app进入后台
 @param context 游戏上下文
 @param seqId 请求序列号
 --]]
function appEnterBackground(context, seqId)
    print_Debug("------- Lua Receive NativeBridge appEnterBackground context = " .. context .. " seqId = " .. seqId)
    SendMessage(MsgID_Native_Bridge_EnterBackground)
    IsAPPInBackground = true
    Audio.pauseAll()
end


----------------------------------------
-- #pragma mark  -- 业务能力

--[[
 游戏从app获取数据回调
 @param context 游戏上下文
 @param seqId 游戏请求时候传的序列号
 @param dataJson json数据
 --]]
function appGetInfoFromAppCallback(context, seqId, dataJson)
    print_Debug("------- Lua Receive NativeBridge appGetInfoFromAppCallback context = " .. context .. " seqId = " .. seqId .. " dataJson = " .. dataJson)
    SendMessage(MsgID_Native_Bridge_APPInfo, context, dataJson, seqId)
end

--[[
 游戏从app获取个人资料回调
 @param context 游戏上下文
 @param seqId 游戏请求时候传的序列号
 @param dataJson json数据
 --]]
function appGetUserInfoCallback(context, seqId, dataJson)
    print_Debug("------- Lua Receive NativeBridge appGetUserInfoCallback context = " .. context .. " seqId = " .. seqId .. " dataJson" .. dataJson)
    SendMessage(MsgID_Native_Bridge_UserInfo,context, dataJson, seqId)
end

--[[
 游戏从app获取游戏信息回调
 @param context 游戏上下文
 @param seqId 游戏请求时候传的序列号
 @param dataJson json数据
 --]]
function appGetGameInfoCallback(context, seqId, dataJson)
    print_Debug("------- Lua Receive NativeBridge appGetGameInfoCallback context = " .. context .. " seqId = " .. seqId .. " dataJson:" .. dataJson)
    SendMessage(MsgID_Native_Bridge_GameInfo, context, dataJson, seqId)
end

--[[
 游戏从app获取设备信息回调
 @param context 游戏上下文
 @param seqId 游戏请求时候传的序列号
 @param dataJson json数据
 --]]
function appGetDeviceInfoCallback(context, seqId, dataJson)
    print_Debug("------- Lua Receive NativeBridge appGetDeviceInfoCallback context = " .. context .. " seqId = " .. seqId .. " dataJson" .. dataJson)
    SendMessage(MsgID_Native_Bridge_DeveiceInfo, dataJson, seqId)
end

--[[
 游戏注册的事件发生时, app回传数据]
 @param context 游戏上下文
 @param seqId 请求序列号
 @param event 事件ID
 @param dataJson 回调数据
 --]]
function appRegisteredEventCallback(context, seqId, event, dataJson)
    print_Debug("------- Lua Receive NativeBridge appRegisteredEventCallback context = " .. context .. " seqId = " .. seqId .. " event" .. event .. " dataJson" .. dataJson)
    SendMessage(MsgID_Native_Bridge_RegisteredEvent, context, event, dataJson, seqId)
end

----------------------------------------
-- #pragma mark  -- 数据存储

--[[
 app保存数据成功回调
 @param context 游戏上下文
 @param savedKey 存储的key
 @param success 是否成功
 --]]
function appSaveLocalDataCallback(context, savedKey, success)
    print_Debug("------- Lua Receive NativeBridge appSaveLocalDataCallback context = " .. context .. " savedKey = " .. savedKey .. " success" .. success)
    SendMessage(MsgID_Native_Bridge_SaveLocalData, savedKey, success, context)
end

--[[
 app读取数据成功回调
 @param context 游戏上下文
 @param loadedKey 读取的key
 @param jsonData 读取的数据
 @param success 是否成功
 --]]
function appLoadDataCallback(context, loadedKey, jsonData, success)
    print_Debug("------- Lua Receive NativeBridge appLoadDataCallback context = " .. loadedKey .. " loadedKey = " .. jsonData .. " jsonData" .. jsonData.. " success" .. success)
    SendMessage(MsgID_Native_Bridge_LoadData, loadedKey, jsonData, success, context)
end

----------------------------------------
-- #pragma mark  -- 文件下载

--[[
 app下载文件回调
 @param context 游戏上下文
 @param fileUrl 下载文件的url
 @param localFilePath 本地目标路径
 @param success 是否成功
 --]]
function appDownloadFileCallback(context, fileUrl, localFilePath, success)
    print_Debug("------- Lua Receive NativeBridge appDownloadFileCallback context = " .. context .. " fileUrl = " .. fileUrl .. " localFilePath" .. localFilePath .. " success" .. success)
    SendMessage(MsgID_Native_Bridge_DownloadFile, context, localFilePath, success)
end

--[[
获取游戏图片数据回调
@param context 游戏上下文
@param imageUrl 请求的图片地址
@param textureKey TextureCache中的textureKey 为nil 或者 "" 表示加载失败
    local sprite = display.newSprite(texture)
--]]
function appGetImageDataCallback(context, imageUrl, textureKey)
    print_Debug("------- Lua Receive NativeBridge appGetImageDataCallback context = " .. context .. " imageUrl = " .. imageUrl .. " textureKey =" .. textureKey)
    if textureKey and textureKey ~="" then
        local texture = cc.Director:getInstance():getTextureCache():getTextureForKey(textureKey)
        if texture and tolua.type(texture) == "cc.Texture2D" then
            SendMessage(MsgID_Native_Bridge_ImageData, context, texture)
        end
    end
end

--[[描述:销毁游戏引擎*
    参数:无
    返回值:无
--]]
function releaseGameEngine()
    print_Debug("------- Lua Receive NativeBridge releaseGameEngine")
    if closeAllWebsocketConnections ~=nil and type(closeAllWebsocketConnections) == 'function' then
        closeAllWebsocketConnections()
    end
    SendMessage(MsgID_Native_Bridge_Release_Game_Engine)
    MsgSystemRelease()
end
