
--
--[[--

 	全局计时器、计划任务

]]
local Scheduler = {}

local schedulerIns = cc.Director:getInstance():getScheduler()

--[[--

计划一个全局帧事件回调，并返回该计划的句柄。

全局帧事件在任何场景中都会执行，因此可以在整个应用程序范围内实现较为精确的全局计时器。

该函数返回的句柄用作 Scheduler.unscheduleGlobal() 的参数，可以取消指定的计划。

@param function 回调函数

@return mixed schedule句柄

]]
function Scheduler.scheduleUpdateGlobal(listener)
    return schedulerIns:scheduleScriptFunc(listener, 0, false)
end

--[[--

计划一个以指定时间间隔执行的全局事件回调，并返回该计划的句柄。

~~~ lua

local function onInterval(dt)
end

-- 每 0.5 秒执行一次 onInterval()
local handle = Scheduler.scheduleGlobal(onInterval, 0.5)

~~~

@param function listener 回调函数
@param number interval 间隔时间

@return mixed schedule句柄

]]
function Scheduler.scheduleGlobal(listener, interval)
    return schedulerIns:scheduleScriptFunc(function( ... )
    	if listener then
    		listener(...)
    	end
    end, interval, false)
end


--[[--

取消一个全局计划

Scheduler.unscheduleGlobal() 的参数就是 Scheduler.scheduleUpdateGlobal() 和 Scheduler.scheduleGlobal() 的返回值。

@param mixed schedule句柄

]]
function Scheduler.unscheduleGlobal(handle)
    schedulerIns:unscheduleScriptEntry(handle)
end

--[[--

计划一个全局延时回调，并返回该计划的句柄。

scheduler.performWithDelayGlobal() 会在等待指定时间后执行一次回调函数，然后自动取消该计划。

@param function listener 回调函数
@param number time 延迟时间

@return mixed schedule句柄

]]
function Scheduler.performWithDelayGlobal(listener, time)
    local handle
    handle = schedulerIns:scheduleScriptFunc(function(...)
        Scheduler.unscheduleGlobal(handle)
        listener(...)
    end, time, false)
    return handle
end

return Scheduler