local LcNetProto = class('LcNetProto')
local _onWebSocketMsgHandler = nil
local _webSocketSendString = nil
local _sbyteFunc = string.byte
local _ssubFunc = string.sub
local _tconcatFunc = table.concat
-----------------socketIO Key Messsage Head------------------------------

local _frame_open_key = _sbyteFunc("0")
local _frame_close_key = _sbyteFunc("1")
local _frame_ping_key = _sbyteFunc("2")
local _frame_pong_key = _sbyteFunc("3")
local _frame_message_key = _sbyteFunc("4")

local _type_binary_event = _sbyteFunc("7")

-------------------------------------------------------------------------

LcNetProtoCallbackType = 
{
    _open_Type = 1,
    _msg_Type = 2,
    _binary_Msg_Type = 3,
}

local _callbackOpenType = LcNetProtoCallbackType._open_Type
local _callbackMsgType = LcNetProtoCallbackType._msg_Type
local _callbackBinaryMsgType = LcNetProtoCallbackType._binary_Msg_Type

function LcNetProto:ctor(url)
    self.objWebSocket = nil
    
    self.callBackFuncs = {}

    self.iBinaryPendingBuffers = 0
    self.szBinaryEventName = nil

    -- proto Name是为了兼容老协议，也是为了方便现在海鸥团队而使用的，仅仅是逻辑的概念
    -- protoIDMapping和protoNameMapping只需要设置一个，另一个被推导出来的。
    -- 查找 proto ID、proto Name 和 pb Name 的对应关系，必须table {proto ID = {proto Name,pb Name},....}, recv时使用proto ID查找proto Name和pb 
    self.protoIDMapping  = nil
    
    -- 查找 proto Name、proto ID 和 pb Name 的对应关系，必须table {proto Name = {proto ID,pb Name}}, send时使用proto Name查找proto ID和pb
    self.protoNameMapping  = nil

    self.defaultSendTextFunc = nil
    self.defaultSendBinaryFunc = nil
end

-- 将WebSocket对象注册给LcNetProto，这样当WebSocket接收到Msg消息(cc.WEBSOCKET_MESSAGE)后将转发给LcNetProto， LcNetProto解析后发给用户层
-- 当发送msg时，默认的情况 LcNetProto会通过此WebSocket对象发送
function LcNetProto:registerToWebSocket(objWebSocket)
    self.objWebSocket = objWebSocket
       
    objWebSocket:registerScriptHandler( function(strBuff) 
        _onWebSocketMsgHandler(self,strBuff)
    end ,cc.WEBSOCKET_MESSAGE)
end

function LcNetProto:registerFun(iType,func)
    if iType == _callbackOpenType or iType == _callbackMsgType or iType == _callbackBinaryMsgType then
        self.callBackFuncs[iType] = func
    end
end

function LcNetProto:SetProtoIDMapping( protoIDMapping )
    local bError = false
    local szError = nil 
    if protoIDMapping == nil then
        bError = true
        szError = "protoIDMapping = nil"
    end
    if not bError and type(protoIDMapping) ~= "table" then
        bError = true
        szError = "protoIDMapping ~= table"
    end
    
    if not bError then
        self.protoNameMapping = {}
        self.protoIDMapping = protoIDMapping

        for k, v in pairs(protoIDMapping) do
            if type(v) == "table" and (#v) == 2 then 
                self.protoNameMapping[v[1]] = {k,v[2]}
            else
                -- error
                szError = "line k = "..k.."is error"
                bError = true
                break
            end  
        end
    end    

    if bError then
        self.protoNameMapping = nil
        self.protoIDMapping = nil
    end

    return (not bError),szError
end 

function LcNetProto:SetProtoNameMapping( protoNameMapping )
    local bError = false
    local szError = nil 
    if protoNameMapping == nil then
        bError = true
        szError = "protoNameMapping = nil"
    end

    if not bError and type(protoNameMapping) ~= "table" then
        bError = true
        szError = "protoNameMapping ~= table"
    end

    if not bError then
        self.protoNameMapping = protoNameMapping
        self.protoIDMapping = {}
        for k, v in pairs(protoNameMapping) do
            if type(v) == "table" and (#v) == 2 then
                self.protoIDMapping[v[1]] = {k,v[2]}
            else
                -- error
                 -- error
                szError = "line k = "..k.."is error"
                bError = true
                break
            end
        end
    end
    return (not bError),szError
end

function LcNetProto:PraseBinaryMsg(iProtoID,strBuff,iOffset)
    if self.protoIDMapping ~= nil then 
        local protoInfor = self.protoIDMapping[iProtoID]
        if protoInfor ~= nil then
           
            local onMsgFunc = nil

            if self.callBackFuncs[_callbackBinaryMsgType] ~= nil then
                onMsgFunc = self.callBackFuncs[_callbackBinaryMsgType]
                local tPb = protobuf.decode(protoInfor[2], strBuff,iOffset)
                onMsgFunc(protoInfor[1],tPb)
            end
        end
    end
end



function LcNetProto:sendOneBinaryData(szProto,strBuffData,sendBinaryFun)
            
    local iProtoId = nil
    local szPBName = nil
        
    if self.protoNameMapping ~= nil then
        local protoInfor =self.protoNameMapping[szProto]

        if protoInfor ~= nil and type(protoInfor) == "table" then
            iProtoId = protoInfor[1]
            szPBName = protoInfor[2]
        end       
    end

    if strBuffData == nil then
        strBuffData = ""
    elseif type(strBuffData) == "table" and szPBName ~= "nil" then 
        strBuffData = protobuf.encode(szPBName, strBuffData)
    end        

    if type(strBuffData) ~= "string" then
        -- error    
        strBuffData = ""
    end
        
    self:sendOneBinaryBuff(iProtoId,strBuffData,sendBinaryFun)
end

local _BinaryHead = _tconcatFunc({string.char(_frame_message_key),string.char(_type_binary_event)})

function LcNetProto:sendOneBinaryBuff(iProtoId,strBuff,sendBinaryFun)
    
    if sendBinaryFun == nil then
        sendBinaryFun = self:GetDefSendBinaryFunc()
    end
    local iHigh = math.floor(iProtoId/256)
    local iLow = iProtoId - (iHigh *256)
    local szSendBuf = _tconcatFunc({_BinaryHead,string.char(iHigh),string.char(iLow),strBuff})

    sendBinaryFun(szSendBuf)
end

function LcNetProto:GetDefSendTextFunc()
    if self.defaultSendTextFunc == nil then
        local objWebSocket = self.objWebSocket
        self.defaultSendTextFunc = function(sendStr)
            if objWebSocket and objWebSocket:getReadyState() == cc.WEBSOCKET_STATE_OPEN then
                objWebSocket:sendString(sendStr)
            else
                if objWebSocket ~= nil then
                    print(':error webSocket state = ', objWebSocket:getReadyState())
                else
                    print(':error webSocket is nil')
                end
            end
        end
    end

    return self.defaultSendTextFunc
end

function LcNetProto:GetDefSendBinaryFunc()
    if self.defaultSendBinaryFunc == nil then
        self.defaultSendBinaryFunc = self:GetDefSendTextFunc()
    end

    return self.defaultSendBinaryFunc
end


_onWebSocketMsgHandler = function(objLcNetProto,strBuff)
    
    if objLcNetProto ~= nil and type(strBuff) == "string" then
        --prase
        local iLen = string.len(strBuff)

        if iLen>0 then
            local frameKey = _sbyteFunc(strBuff,1)
            if frameKey == _frame_open_key then
        
                local openFunc = objLcNetProto.callBackFuncs[_callbackOpenType]
                if openFunc ~= nil  then
                    openFunc()
                end
                
            elseif frameKey == _frame_close_key then

            elseif frameKey == _frame_ping_key then

            elseif frameKey == _frame_pong_key then

            elseif frameKey == _frame_message_key and iLen>1 then
                -- msg
                local typeKey = _sbyteFunc(strBuff,2)
            
                -- 47ID(int16)buff
                if typeKey == _type_binary_event and iLen>3 then
                    -- find "-" cal conut            
                    -- int 16
                    local iProtoID = _sbyteFunc(strBuff,3)*256 + _sbyteFunc(strBuff,4)
                    objLcNetProto:PraseBinaryMsg(iProtoID,strBuff,5)
                
                end

            end
        end
    end

end

-- test
function LcNetProto:TestDecodeData(strBuff)
    _onWebSocketMsgHandler(self,strBuff)
end


return LcNetProto
